@@ -55,7 +55,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)res_debug.c	8.1 (Berkeley) 6/4/93";
-static char rcsid[] = "res_debug.c,v 1.1.1.1 1995/02/18 05:33:20 hjl Exp";
+static char rcsid[] = "res_debug.c,v 1.2 1995/05/06 14:12:41 hjl Exp";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
@@ -64,61 +64,37 @@ static char rcsid[] = "res_debug.c,v 1.1.1.1 1995/02/18 05:33:20 hjl Exp";
 #include <arpa/nameser.h>
 
 #include <stdio.h>
+#include <netdb.h>
 #include <resolv.h>
-#if defined(BSD) && (BSD >= 199103)
-# include <string.h>
-#else
-# include "inetprivate.h"
+#include <string.h>
+
+#include "res_options.h"
+
+extern const char *_res_opcodes[];
+extern const char *_res_resultcodes[];
+
+#ifdef __ELF__
+#pragma weak fp_query = __fp_query
+#pragma weak fp_nquery = __fp_nquery
+#pragma weak fp_resstat = __fp_resstat
+#pragma weak p_cdname = __p_cdname
+#pragma weak p_cdnname = __p_cdnname
+#pragma weak p_class = __p_class
+#pragma weak p_fqname = __p_fqname
+#pragma weak p_option = __p_option
+#pragma weak p_query = __p_query
+#pragma weak p_rr = __p_rr
+#pragma weak p_time = __p_time
+#pragma weak p_type = __p_type
 #endif
 
-#if defined(USE_OPTIONS_H)
-# include "res_opts.h"
-#endif
-
-const char *_res_opcodes[] = {
-	"QUERY",
-	"IQUERY",
-	"CQUERYM",
-	"CQUERYU",
-	"4",
-	"5",
-	"6",
-	"7",
-	"8",
-	"UPDATEA",
-	"UPDATED",
-	"UPDATEDA",
-	"UPDATEM",
-	"UPDATEMA",
-	"ZONEINIT",
-	"ZONEREF",
-};
-
-const char *_res_resultcodes[] = {
-	"NOERROR",
-	"FORMERR",
-	"SERVFAIL",
-	"NXDOMAIN",
-	"NOTIMP",
-	"REFUSED",
-	"6",
-	"7",
-	"8",
-	"9",
-	"10",
-	"11",
-	"12",
-	"13",
-	"14",
-	"NOCHANGE",
-};
-
-static char retbuf[16];
-
+/* XXX: we should use getservbyport() instead. */
 static const char *
 dewks(wks)
 	int wks;
 {
+	static char nbuf[20];
+
 	switch (wks) {
 	case 5: return "rje";
 	case 7: return "echo";
@@ -166,14 +142,17 @@ dewks(wks)
 	case 161: return "snmp";
 	case 162: return "snmp-trap";
 	case 170: return "print-srv";
-	default: (void) sprintf(retbuf, "%d", wks); return (retbuf);
+	default: (void) sprintf(nbuf, "%d", wks); return (nbuf);
 	}
 }
 
+/* XXX: we should use getprotobynumber() instead. */
 static const char *
 deproto(protonum)
 	int protonum;
 {
+	static char nbuf[20];
+
 	switch (protonum) {
 	case 1: return "icmp";
 	case 2: return "igmp";
@@ -187,13 +166,13 @@ deproto(protonum)
 	case 12: return "pup";
 	case 16: return "chaos";
 	case 17: return "udp";
-	default: (void) sprintf(retbuf, "%d", protonum); return (retbuf);
+	default: (void) sprintf(nbuf, "%d", protonum); return (nbuf);
 	}
 }
 
 static const u_char *
-do_rrset(msg, cp, cnt, pflag, file, hs)
-	int cnt, pflag;
+do_rrset(msg, len, cp, cnt, pflag, file, hs)
+	int cnt, pflag, len;
 	const u_char *cp, *msg;
 	const char *hs;
 	FILE *file;
@@ -210,8 +189,19 @@ do_rrset(msg, cp, cnt, pflag, file, hs)
 		    ((sflag) && (_res.pfcode & RES_PRF_HEAD1)))
 			fprintf(file, hs);
 		while (--n >= 0) {
-			cp = p_rr(cp, msg, file);
-			if ((cp - msg) > PACKETSZ)
+			if ((!_res.pfcode) || sflag) {
+				cp = __p_rr(cp, msg, file);
+			} else {
+				unsigned int dlen;
+				cp += __dn_skipname(cp, cp + MAXCDNAME);
+				cp += INT16SZ;
+				cp += INT16SZ;
+				cp += INT32SZ;
+				dlen = _getshort((u_char*)cp);
+				cp += INT16SZ;
+				cp += dlen;
+			}
+			if ((cp - msg) > len)
 				return (NULL);
 		}
 		if ((!_res.pfcode) ||
@@ -228,17 +218,6 @@ __p_query(msg)
 	__fp_query(msg, stdout);
 }
 
-#ifdef ultrix
-/* ultrix 4.0's packaging has some icky packaging.  alias for it here.
- * there is more junk of this kind over in res_comp.c.
- */
-void
-p_query(msg)
-	const u_char *msg;
-{
-	__p_query(msg);
-}
-#endif
 
 /*
  * Print the current options.
@@ -274,6 +253,9 @@ __fp_nquery(msg, len, file)
 	register const HEADER *hp;
 	register int n;
 
+	if ((_res.options & RES_INIT) == 0 && res_init() == -1)
+		return;
+
 #define TruncTest(x) if (x >= endMark) goto trunc
 #define	ErrorTest(x) if (x == NULL) goto error
 
@@ -290,9 +272,10 @@ __fp_nquery(msg, len, file)
 			ntohs(hp->id));
 		putc('\n', file);
 	}
-	putc(';', file);
+	if ((!_res.pfcode) || (_res.pfcode & RES_PRF_HEADX))
+		putc(';', file);
 	if ((!_res.pfcode) || (_res.pfcode & RES_PRF_HEAD2)) {
-		fprintf(file,"; flags:");
+		fprintf(file, "; flags:");
 		if (hp->qr)
 			fprintf(file, " qr");
 		if (hp->aa)
@@ -303,8 +286,6 @@ __fp_nquery(msg, len, file)
 			fprintf(file, " rd");
 		if (hp->ra)
 			fprintf(file, " ra");
-		if (hp->pr)
-			fprintf(file, " pr");
 	}
 	if ((!_res.pfcode) || (_res.pfcode & RES_PRF_HEAD1)) {
 		fprintf(file, "; Ques: %d", ntohs(hp->qdcount));
@@ -321,11 +302,23 @@ __fp_nquery(msg, len, file)
 	 */
 	if (n = ntohs(hp->qdcount)) {
 		if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
-			fprintf(file,";; QUESTIONS:\n");
+			fprintf(file, ";; QUESTIONS:\n");
 		while (--n >= 0) {
-			fprintf(file,";;\t");
+			if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
+				fprintf(file, ";;\t");
 			TruncTest(cp);
-			cp = p_cdname(cp, msg, file);
+			if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
+				cp = p_cdnname(cp, msg, len, file);
+			else {
+				int n;
+				char name[MAXDNAME];
+
+				if ((n = dn_expand(msg, msg+len, cp, name,
+						sizeof name)) < 0)
+					cp = NULL;
+				else
+					cp += n;
+			}
 			ErrorTest(cp);
 			TruncTest(cp);
 			if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
@@ -337,14 +330,15 @@ __fp_nquery(msg, len, file)
 				fprintf(file, ", class = %s\n",
 					__p_class(_getshort((u_char*)cp)));
 			cp += INT16SZ;
-			putc('\n', file);
+			if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
+				putc('\n', file);
 		}
 	}
 	/*
 	 * Print authoritative answer records
 	 */
 	TruncTest(cp);
-	cp = do_rrset(msg, cp, hp->ancount, RES_PRF_ANS, file,
+	cp = do_rrset(msg, len, cp, hp->ancount, RES_PRF_ANS, file,
 		      ";; ANSWERS:\n");
 	ErrorTest(cp);
 
@@ -352,7 +346,7 @@ __fp_nquery(msg, len, file)
 	 * print name server records
 	 */
 	TruncTest(cp);
-	cp = do_rrset(msg, cp, hp->nscount, RES_PRF_AUTH, file,
+	cp = do_rrset(msg, len, cp, hp->nscount, RES_PRF_AUTH, file,
 		      ";; AUTHORITY RECORDS:\n");
 	ErrorTest(cp);
 
@@ -360,7 +354,7 @@ __fp_nquery(msg, len, file)
 	/*
 	 * print additional records
 	 */
-	cp = do_rrset(msg, cp, hp->arcount, RES_PRF_ADD, file,
+	cp = do_rrset(msg, len, cp, hp->arcount, RES_PRF_ADD, file,
 		      ";; ADDITIONAL RECORDS:\n");
 	ErrorTest(cp);
 	return;
@@ -414,7 +408,7 @@ __p_fqname(cp, msg, file)
 	FILE *file;
 {
 	char name[MAXDNAME];
-	int n, len;
+	int n;
 
 	if ((n = dn_expand(msg, cp + MAXCDNAME, cp, name, sizeof name)) < 0)
 		return (NULL);
@@ -442,6 +436,10 @@ __p_rr(cp, msg, file)
 	u_int32_t tmpttl, t;
 	int lcnt;
 
+	if ((_res.options & RES_INIT) == 0 && res_init() == -1) {
+		h_errno = NETDB_INTERNAL;
+		return (NULL);
+	}
 	if ((cp = p_fqname(cp, msg, file)) == NULL)
 		return (NULL);			/* compression error */
 	type = _getshort((u_char*)cp);
@@ -454,7 +452,7 @@ __p_rr(cp, msg, file)
 	cp += INT16SZ;
 	cp1 = cp;
 	if ((!_res.pfcode) || (_res.pfcode & RES_PRF_TTLID))
-		fprintf(file, "\t%lu", tmpttl);
+		fprintf(file, "\t%lu", (u_long)tmpttl);
 	if ((!_res.pfcode) || (_res.pfcode & RES_PRF_CLASS))
 		fprintf(file, "\t%s", __p_class(class));
 	fprintf(file, "\t%s", __p_type(type));
@@ -468,7 +466,7 @@ __p_rr(cp, msg, file)
 		case C_HS:
 			bcopy(cp, (char *)&inaddr, INADDRSZ);
 			if (dlen == 4) {
-				fprintf(file,"\t%s", inet_ntoa(inaddr));
+				fprintf(file, "\t%s", inet_ntoa(inaddr));
 				cp += dlen;
 			} else if (dlen == 7) {
 				char *address;
@@ -496,55 +494,75 @@ __p_rr(cp, msg, file)
 	case T_NS:
 	case T_PTR:
 		putc('\t', file);
-		cp = p_fqname(cp, msg, file);
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		break;
 
 	case T_HINFO:
 	case T_ISDN:
+		cp2 = cp + dlen;
 		if (n = *cp++) {
-			fprintf(file,"\t%.*s", n, cp);
+			fprintf(file, "\t%.*s", n, cp);
 			cp += n;
 		}
-		if (n = *cp++) {
-			fprintf(file,"\t%.*s", n, cp);
+		if ((cp < cp2) && (n = *cp++)) {
+			fprintf(file, "\t%.*s", n, cp);
 			cp += n;
-		}
+		} else if (type == T_HINFO)
+			fprintf(file, "\n;; *** Warning *** OS-type missing");
 		break;
 
 	case T_SOA:
 		putc('\t', file);
-		cp = p_fqname(cp, msg, file);	/* origin */
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		putc(' ', file);
-		cp = p_fqname(cp, msg, file);	/* mail addr */
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		fputs(" (\n", file);
 		t = _getlong((u_char*)cp);  cp += INT32SZ;
-		fprintf(file,"\t\t\t%lu\t; serial\n", t);
+		fprintf(file, "\t\t\t%lu\t; serial\n", (u_long)t);
 		t = _getlong((u_char*)cp);  cp += INT32SZ;
-		fprintf(file,"\t\t\t%lu\t; refresh (%s)\n", t, __p_time(t));
+		fprintf(file, "\t\t\t%lu\t; refresh (%s)\n",
+			(u_long)t, __p_time(t));
 		t = _getlong((u_char*)cp);  cp += INT32SZ;
-		fprintf(file,"\t\t\t%lu\t; retry (%s)\n", t, __p_time(t));
+		fprintf(file, "\t\t\t%lu\t; retry (%s)\n",
+			(u_long)t, __p_time(t));
 		t = _getlong((u_char*)cp);  cp += INT32SZ;
-		fprintf(file,"\t\t\t%lu\t; expire (%s)\n", t, __p_time(t));
+		fprintf(file, "\t\t\t%lu\t; expire (%s)\n",
+			(u_long)t, __p_time(t));
 		t = _getlong((u_char*)cp);  cp += INT32SZ;
-		fprintf(file,"\t\t\t%lu )\t; minimum (%s)", t, __p_time(t));
+		fprintf(file, "\t\t\t%lu )\t; minimum (%s)",
+			(u_long)t, __p_time(t));
 		break;
 
 	case T_MX:
 	case T_AFSDB:
 	case T_RT:
-		fprintf(file,"\t%d ", _getshort((u_char*)cp));
+		fprintf(file, "\t%d ", _getshort((u_char*)cp));
 		cp += INT16SZ;
-		cp = p_fqname(cp, msg, file);
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		break;
 
-  	case T_TXT:
+	case T_PX:
+		fprintf(file, "\t%d ", _getshort((u_char*)cp));
+		cp += INT16SZ;
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
+		putc(' ', file);
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
+		break;
+
+	case T_TXT:
 	case T_X25:
 		(void) fputs("\t\"", file);
 		cp2 = cp1 + dlen;
 		while (cp < cp2) {
 			if (n = (unsigned char) *cp++) {
 				for (c = n; c > 0 && cp < cp2; c--)
-					if (*cp == '\n') {
+					if ((*cp == '\n') || (*cp == '"')) {
 					    (void) putc('\\', file);
 					    (void) putc(*cp++, file);
 					} else
@@ -552,19 +570,21 @@ __p_rr(cp, msg, file)
 			}
 		}
 		putc('"', file);
-  		break;
+		break;
 
-  	case T_NSAP:
+	case T_NSAP:
 		(void) fprintf(file, "\t%s", inet_nsap_ntoa(dlen, cp, NULL));
 		cp += dlen;
-  		break;
+		break;
 
 	case T_MINFO:
 	case T_RP:
 		putc('\t', file);
-		cp = p_fqname(cp, msg, file);
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		putc(' ', file);
-		cp = p_fqname(cp, msg, file);
+		if ((cp = p_fqname(cp, msg, file)) == NULL)
+			return (NULL);
 		break;
 
 	case T_UINFO:
@@ -576,7 +596,7 @@ __p_rr(cp, msg, file)
 	case T_UID:
 	case T_GID:
 		if (dlen == 4) {
-			fprintf(file,"\t%u", _getlong((u_char*)cp));
+			fprintf(file, "\t%u", _getlong((u_char*)cp));
 			cp += INT32SZ;
 		}
 		break;
@@ -595,7 +615,7 @@ __p_rr(cp, msg, file)
 		while (cp < cp1 + dlen) {
 			c = *cp++;
 			do {
- 				if (c & 0200) {
+				if (c & 0200) {
 					if (lcnt == 0) {
 						fputs("\n\t\t\t", file);
 						lcnt = 5;
@@ -604,7 +624,7 @@ __p_rr(cp, msg, file)
 					putc(' ', file);
 					lcnt--;
 				}
- 				c <<= 1;
+				c <<= 1;
 			} while (++n & 07);
 		}
 		putc(')', file);
@@ -628,7 +648,7 @@ __p_rr(cp, msg, file)
 #endif /* ALLOW_T_UNSPEC */
 
 	default:
-		fprintf(file,"\t?%d?", type);
+		fprintf(file, "\t?%d?", type);
 		cp += dlen;
 	}
 #if 0
@@ -637,15 +657,13 @@ __p_rr(cp, msg, file)
 	putc('\n', file);
 #endif
 	if (cp - cp1 != dlen) {
-		fprintf(file,";; packet size error (found %d, dlen was %d)\n",
+		fprintf(file, ";; packet size error (found %d, dlen was %d)\n",
 			cp - cp1, dlen);
 		cp = NULL;
 	}
 	return (cp);
 }
 
-static	char nbuf[40];
-
 /*
  * Return a string for the type
  */
@@ -653,6 +671,8 @@ const char *
 __p_type(type)
 	int type;
 {
+	static char nbuf[20];
+
 	switch (type) {
 	case T_A:	return "A";
 	case T_NS:	return "NS";
@@ -668,12 +688,19 @@ __p_type(type)
 	case T_MINFO:	return "MINFO";
 	case T_MX:	return "MX";
 	case T_TXT:	return "TXT";
-	case T_NSAP:	return "NSAP";
 	case T_RP:	return "RP";
 	case T_AFSDB:	return "AFSDB";
 	case T_X25:	return "X25";
 	case T_ISDN:	return "ISDN";
 	case T_RT:	return "RT";
+	case T_NSAP:	return "NSAP";
+	case T_NSAP_PTR: return "NSAP_PTR";
+	case T_SIG:	return "SIG";
+	case T_KEY:	return "KEY";
+	case T_PX:	return "PX";
+	case T_GPOS:	return "GPOS";
+	case T_AAAA:	return "AAAA";
+	case T_LOC:	return "LOC";
 	case T_AXFR:	return "AXFR";
 	case T_MAILB:	return "MAILB";
 	case T_MAILA:	return "MAILA";
@@ -695,10 +722,12 @@ const char *
 __p_class(class)
 	int class;
 {
+	static char nbuf[20];
+
 	switch (class) {
-	case C_IN:	return("IN");
-	case C_HS:	return("HS");
-	case C_ANY:	return("ANY");
+	case C_IN:	return "IN";
+	case C_HS:	return "HS";
+	case C_ANY:	return "ANY";
 	default:	(void)sprintf(nbuf, "%d", class); return (nbuf);
 	}
 }
@@ -710,12 +739,14 @@ const char *
 __p_option(option)
 	u_long option;
 {
+	static char nbuf[40];
+
 	switch (option) {
 	case RES_INIT:		return "init";
 	case RES_DEBUG:		return "debug";
-	case RES_AAONLY:	return "aaonly";
+	case RES_AAONLY:	return "aaonly(unimpl)";
 	case RES_USEVC:		return "usevc";
-	case RES_PRIMARY:	return "primry";
+	case RES_PRIMARY:	return "primry(unimpl)";
 	case RES_IGNTC:		return "igntc";
 	case RES_RECURSE:	return "recurs";
 	case RES_DEFNAMES:	return "defnam";
@@ -723,7 +754,8 @@ __p_option(option)
 	case RES_DNSRCH:	return "dnsrch";
 	case RES_INSECURE1:	return "insecure1";
 	case RES_INSECURE2:	return "insecure2";
-	default:		sprintf(nbuf, "?0x%x?", option); return nbuf;
+	default:		sprintf(nbuf, "?0x%lx?", (u_long)option);
+				return (nbuf);
 	}
 }
 
@@ -734,6 +766,7 @@ char *
 __p_time(value)
 	u_int32_t value;
 {
+	static char nbuf[40];
 	int secs, mins, hours, days;
 	register char *p;
 
